<?php 
class WCOSM_Product
{
	public function __construct()
	{
		if(is_admin())
		{
			add_action('wp_ajax_wcosm_get_stock_values', array(&$this, 'ajax_get_stock_values'));
			add_action('wp_ajax_wcosm_save_modified_data', array(&$this, 'ajax_save_stock_values'));
		}
	}
	public function ajax_save_stock_values()
	{
		global $wcosm_wpml_model;
		$data_to_save = isset($_POST['data']) ? json_decode(stripcslashes ($_POST['data'])): array();
		//wcosm_var_dump($data_to_save);
	
		//instock - outofstock
		foreach($data_to_save as $item)
		{
			if(is_object($item))
			{
				update_post_meta($item->id, '_stock', $item->value);
				do_action('wplister_revise_inventory_status', $item->id );
				$previous_stock_status = get_post_meta($item->id, '_stock_status', true);
				//wcosm_var_dump($previous_stock_status);
				if($item->value > 0)
				{
					update_post_meta($item->id, '_stock_status', 'instock');
					if($previous_stock_status == 'outofstock')
						do_action('wplister_relist_item', $item->id );
				}
				else
				{
					update_post_meta($item->id, '_stock_status', 'outofstock');
				}
				
			}
			//WPML
			$item->type = $item->type == "variation" ? "product_variation" : "product";
			$translations = $wcosm_wpml_model->get_all_translation_ids($item->id, $item->type);
			//wcosm_var_dump($translations);
			if($translations != false)
				foreach($translations as /* $langkey =>  */$translation)
				{
					//if($translation->language_code != $wcosm_wpml_model->get_default_lang())
					{
						update_post_meta($translation, '_stock', $item->value);
						do_action('wplister_revise_inventory_status', $translation );
						$previous_stock_status = get_post_meta($translation, '_stock_status', true);
						if($item->value > 0)
						{
							update_post_meta($translation, '_stock_status', 'instock');
							if($previous_stock_status == 'outofstock')
								do_action('wplister_relist_item', $translation );
						}
						else
						{
							update_post_meta($translation, '_stock_status', 'outofstock');
						}
						
					}
				}
		}
		wp_die();
	}
	public function ajax_get_stock_values()
	{
		$max_results_num = isset($_POST['max_results_num']) ? $_POST['max_results_num'] : 'all';
		$view_type = isset($_POST['view_type']) ? $_POST['view_type'] : "threshold_level";
		$result = $this->get_product_by_stock_levels($max_results_num, $view_type);
		echo json_encode($result);
		wp_die();
	}
	public function get_product_variations_by_warning_level($product_ids, $additional_ids_string=null, $warning_level = 10, $exclude = false)
	{
		global $wpdb, $wcosm_wpml_model;
		$id_string = "";
		$exclude_condition = $exclude ? " NOT ": "";
		$join_id_condition = $exclude ? " AND " : " OR ";
		//$exclude_condition = "";
		
		if(is_array($product_ids) && !empty($product_ids))
			foreach($product_ids as $product)
				$id_string .= $id_string != "" ? ",".$product : $product;
			
		if(isset($additional_ids_string))
			$id_string .=  $id_string != "" ? ",".$additional_ids_string : $additional_ids_string;
		
		if($id_string == "")
		{
			$id_string = -1;
		}
		$wpdb->query('SET group_concat_max_len=5000000'); 	
		$wpdb->query('SET SQL_BIG_SELECTS=1');
		$query = "SELECT products.ID, products.post_title,  products.post_parent, products.post_type, stock.meta_value AS stock_value, 
					     stock_status.meta_value AS stock_status, sku.meta_value AS sku, GROUP_CONCAT(product_meta.meta_key) AS attribute_name, 
						 GROUP_CONCAT(product_meta.meta_value) AS attribute_value, product_duplicate_translation.meta_value AS translation_original_id
				  FROM {$wpdb->posts} AS products 
				  INNER JOIN {$wpdb->postmeta} AS manage_stock ON manage_stock.post_id = products.ID
				  INNER JOIN {$wpdb->postmeta} AS stock ON stock.post_id = products.ID
				  INNER JOIN {$wpdb->postmeta} AS stock_status ON stock_status.post_id = products.ID					  
				  INNER JOIN {$wpdb->postmeta} AS sku ON sku.post_id = products.ID 
				  INNER JOIN {$wpdb->postmeta} AS product_meta ON product_meta.post_id = products.ID 
				  
				  LEFT JOIN {$wpdb->postmeta} AS product_duplicate_translation ON product_duplicate_translation.post_id = products.ID AND product_duplicate_translation.meta_key = '_wcml_duplicate_of_variation' 

				 WHERE".
				//  (products.post_type = 'product' OR products.post_type = 'product_variation')
				//   AND " 
				"  manage_stock.meta_key = '_manage_stock' 
				  AND (products.post_parent {$exclude_condition} IN ({$id_string} ) {$join_id_condition} products.ID {$exclude_condition} IN ({$id_string} ))
				  AND manage_stock.meta_value = 'yes' 
				  AND stock_status.meta_key = '_stock_status' 
				  AND stock.meta_key = '_stock' 
				  AND sku.meta_key = '_sku' 
				  AND products.post_type = 'product_variation' 
				  AND product_meta.meta_key LIKE '%attribute_%' 
				  AND stock.meta_value <= {$warning_level} 
				  AND products.post_status = 'publish'
				  GROUP BY products.ID ";
					  
		//wcosm_var_dump($query);
		$result = $wpdb->get_results($query, ARRAY_A);
		//wcosm_var_dump($result);
		if(isset($result))
			foreach($result as $key => $item)
			{
				$name = $this->get_variation_complete_name($result[$key]['ID']);
				
				if($name == false || $result[$key]['translation_original_id'] != null /* || $wcosm_wpml_model->is_item_a_translation($result[$key]['ID'], 'product_variation') */) //WPML: is a translation -> discard
					unset($result[$key]);
				else
					$result[$key]['post_title'] = $name;
			}
		return isset($result) ? $result : array();
	}
	public function get_product_by_stock_levels($max_items = "all", $view_type = "threshold_level")
	{
		global $wcosm_option_model, $wpdb, $wcosm_wpml_model;
		$all_thresholds = $wcosm_option_model->get_all_threshold_levels();
		$porducts_under_custom_warning_level = array();
		$products_under_default_warning_level = array();
		$ids_to_exclude = array();
		$wcosm_wpml_model->switch_to_default_language();
		
		$max_items = $max_items == "all" ? "" : " LIMIT ".$max_items;
		
		//_stock_status: instock
		$shared_query = "SELECT products.ID, products.post_title, products.post_type, stock.meta_value AS stock_value, 
								stock_status.meta_value AS stock_status, sku.meta_value AS sku, GROUP_CONCAT(term.name) AS category, 
								type_term.name AS product_type, product_duplicate_translation.meta_value AS translation_original_id
					  FROM {$wpdb->posts} AS products 
					  INNER JOIN {$wpdb->postmeta} AS manage_stock ON manage_stock.post_id = products.ID
					  INNER JOIN {$wpdb->postmeta} AS stock ON stock.post_id = products.ID
					  INNER JOIN {$wpdb->postmeta} AS stock_status ON stock_status.post_id = products.ID
					  INNER JOIN {$wpdb->postmeta} AS sku ON sku.post_id = products.ID 
					  
					  LEFT JOIN {$wpdb->term_relationships} AS category_relationship ON (products.ID = category_relationship.object_id) 
					  LEFT JOIN {$wpdb->term_taxonomy} AS taxonomy ON category_relationship.term_taxonomy_id = taxonomy.term_taxonomy_id AND taxonomy.taxonomy = 'product_cat' 
					  LEFT JOIN {$wpdb->terms} AS term ON (term.term_id = taxonomy.term_id) 
					  
					  LEFT JOIN {$wpdb->postmeta} AS product_duplicate_translation ON product_duplicate_translation.post_id = products.ID AND product_duplicate_translation.meta_key = '_icl_lang_duplicate_of' 

					  INNER JOIN {$wpdb->term_relationships} AS type_relationship ON (products.ID = type_relationship.object_id) 
					  INNER JOIN {$wpdb->term_taxonomy} AS type_taxonomy ON type_relationship.term_taxonomy_id = type_taxonomy.term_taxonomy_id 
					  INNER JOIN {$wpdb->terms} AS type_term ON (type_term.term_id = type_taxonomy.term_id) 
					    
					 WHERE".
					//  (products.post_type = 'product' OR products.post_type = 'product_variation')
					//   AND " 
					"  manage_stock.meta_key = '_manage_stock' 
					  AND type_taxonomy.taxonomy = 'product_type'
					  AND manage_stock.meta_value = 'yes' 
					  AND stock_status.meta_key = '_stock_status' 
					  AND sku.meta_key = '_sku' 
					  AND stock.meta_key = '_stock' ";
		//wcosm_var_dump($all_thresholds);
		/* Format: 
			array(2) {
			  ["default"]=>
			  string(2) "11"
			  ['thresholds'] =>
				  [0]=>
				  array(5) {
					["warning_level"]=>
					string(2) "12"
					["select_strategy"]=>  //all / except
					string(3) "all"
					["products"]=>
					string(0) ""
					["categories"]=>
					array(1) {
					  [0]=>
					  int(10)
					}
					 ["email_notification"]=>
					string(3) "yes"
					["categories_children"]=>
					string(13) "selected_only" //selected_only / all_children
			  }
			}
		*/
		if($view_type == "threshold_level")
			foreach($all_thresholds['thresholds'] as $current_threshold)
			{
				//$current_threshold['select_strategy'] = "all";
				$except_suffix = $current_threshold['select_strategy'] == 'all' ? "" : " NOT ";
				$conditions_operator = " OR ";//$current_threshold['select_strategy'] == 'all' ? " OR " : " AND ";
				//$selected_products =  $current_threshold['products'] != "" ? "AND (products.ID ".$except_suffix." IN (".implode(",", $current_threshold['products']).")" : "";
				$additional_products_by_seleceted_categories_query = "";
				$additional_products_by_seleceted_categories = null;
				
				//products by category
				if(!empty($current_threshold['categories']))
				{
					$additional_categories_ids = array();
					if($current_threshold["categories_children"] == 'all_children')
					{
						foreach($current_threshold["categories"] as $current_category)
						{
							$args = array(
									'type'                     => 'post',
									'child_of'                 => $current_category,
									'parent'                   => '',
									'orderby'                  => 'name',
									'order'                    => 'ASC',
									'hide_empty'               => 1,
									'hierarchical'             => 1,
									'exclude'                  => '',
									'include'                  => '',
									'number'                   => '',
									'taxonomy'                 => 'product_cat',
									'pad_counts'               => false

								); 

								$categories = get_categories( $args );
								//wcosm_var_dump($categories);
								foreach($categories as $result)
									$additional_categories_ids[] = $result->term_id;
						}
					}
					
					if(!empty($additional_categories_ids))
						$current_threshold["categories"] = array_merge($current_threshold["categories"],$additional_categories_ids);
					
					$wpdb->query('SET group_concat_max_len=5000000'); 
					$wpdb->query('SET SQL_BIG_SELECTS=1');
					$get_prdocuts_from_categories_query = " SELECT DISTINCT GROUP_CONCAT(products.ID) as product_ids
											 FROM {$wpdb->posts} AS products 
											 INNER JOIN {$wpdb->term_relationships} AS term_rel ON term_rel.object_id = products.ID
											 INNER JOIN {$wpdb->term_taxonomy} AS term_tax ON term_tax.term_taxonomy_id = term_rel.term_taxonomy_id 
											 INNER JOIN {$wpdb->terms} AS terms ON terms.term_id = term_tax.term_id
											 WHERE  terms.term_id IN ('" . implode( "','", $current_threshold["categories"]). "')  
											 AND term_tax.taxonomy = 'product_cat' "; // GROUP BY terms.term_id;
				
					//wcosm_var_dump($get_prdocuts_from_categories_query);
					//recover product by category
					$additional_products_by_seleceted_categories = $wpdb->get_results($get_prdocuts_from_categories_query,ARRAY_A);
					//wcosm_var_dump($additional_products_by_seleceted_categories);
					$additional_products_by_seleceted_categories = isset($additional_products_by_seleceted_categories) ? array_shift($additional_products_by_seleceted_categories) : null;
					//wcosm_var_dump($additional_products_by_seleceted_categories);
				}
				
				//useless additional query => much better to merge $current_threshold['products'] with $additional_products_by_seleceted_categories['product_ids']
				//and then prepare one query
				if(isset($additional_products_by_seleceted_categories) && isset($additional_products_by_seleceted_categories['product_ids']) && !empty($additional_products_by_seleceted_categories['product_ids']) )
				{
					/* $additional_products_by_seleceted_categories = $additional_products_by_seleceted_categories['product_ids'];
					$additional_products_by_seleceted_categories_query = $except_suffix." IN (". $additional_products_by_seleceted_categories.")"; */
					$current_threshold['products'] = !empty($current_threshold['products']) ? array_merge($current_threshold['products'], explode(",",$additional_products_by_seleceted_categories['product_ids'])) :  explode(",",$additional_products_by_seleceted_categories['product_ids']);
					//wcosm_var_dump(count($current_threshold['products']));
				}
				//wcosm_var_dump($current_threshold['products']);
				$selected_products =  is_array($current_threshold['products']) && !empty($current_threshold['products']) ? "AND products.ID ".$except_suffix." IN (".implode(",", $current_threshold['products']).")" : "";
				//wcosm_var_dump($selected_products);
				
				/* //post_parent
				$use_last_parentesis = true;
				if($selected_products != "" && $additional_products_by_seleceted_categories_query != "")
					$selected_products .= $conditions_operator." products.ID ".$additional_products_by_seleceted_categories_query;
				else if($additional_products_by_seleceted_categories_query != "")
				{
					if($selected_products == "")
						$use_last_parentesis = false;
					$selected_products .= " AND products.ID ".$additional_products_by_seleceted_categories_query;
				} 
				
				if($use_last_parentesis && $selected_products != "")
					$selected_products .= ")";
				*/
				
				$query = $shared_query." 
						  AND stock.meta_value <= {$current_threshold['warning_level']} 
						  ".$selected_products." GROUP BY products.ID ".$max_items;
				//wcosm_var_dump($query);
				//recover all products (specified in options and recovere by categories specified in options)
				//wcosm_var_dump($query);
				//$result = null;
				
				//In this way lower rules have higher priority
				/* if(isset($additional_products_by_seleceted_categories) && is_array($additional_products_by_seleceted_categories))
					foreach($additional_products_by_seleceted_categories as $temp_product)
					{
						if(isset($porducts_under_custom_warning_level[$temp_product]))
							unset($porducts_under_custom_warning_level[$temp_product]); 
					} */
				if($current_threshold['products'] != "")
					foreach($current_threshold['products'] as $temp_product) 
					{
						if(isset($porducts_under_custom_warning_level[$temp_product]))
							unset($porducts_under_custom_warning_level[$temp_product]); 
					}
				
				$wpdb->query('SET group_concat_max_len=5000000'); 	
				$wpdb->query('SET SQL_BIG_SELECTS=1');
				//$result = isset($selected_products['products']) && $selected_products['products'] != "" ? $wpdb->get_results($query,ARRAY_A) : null; ?????
				$result = isset($current_threshold['products']) && $current_threshold['products'] != "" ? $wpdb->get_results($query,ARRAY_A) : null; 
				//$result =  $wpdb->get_results($query,ARRAY_A);
				
				if(isset($result))
				{
					foreach($result as $product)
					{
						
						if($product['translation_original_id'] == null && ($current_threshold['select_strategy'] == 'all'  || ($current_threshold['select_strategy'] == 'except'  && !$wcosm_wpml_model->is_item_a_translation($product['ID']) ))) //WPML
						{
							$porducts_under_custom_warning_level[$product['ID']] = $product;
						}
					}
				}
				
				{
					$ids_to_process = $current_threshold['select_strategy'] == 'all' ? $current_threshold['products'] : $this->get_complementry_ids($current_threshold['products']);
					foreach((array)$ids_to_process as $original_selected_prodcut_id)
							$ids_to_exclude[] = $original_selected_prodcut_id;
				}
			
				$exclude_variant = $current_threshold['select_strategy'] == 'all' ? false : true;
				//$variations = $this->get_product_variations_by_warning_level($current_threshold['products'], $additional_products_by_seleceted_categories, $current_threshold['warning_level'], $exclude_variant);
				$variations = $this->get_product_variations_by_warning_level($current_threshold['products'], null, $current_threshold['warning_level'], $exclude_variant);
				$parent_variation_ids = "";
				$parent_to_variations = array();
				foreach($variations as $variation)
				{
					$variation['product_type'] = "variation";
					$parent_variation_ids .= $parent_variation_ids != "" ? ",".$variation['post_parent'] : $variation['post_parent'];
					if(!isset($parent_to_variations[$variation['post_parent']]))
						$parent_to_variations[$variation['post_parent']] = array();
					$parent_to_variations[$variation['post_parent']][] = $variation['ID'];
					
					$porducts_under_custom_warning_level[$variation['ID']] = $variation;
					
					$ids_to_exclude[] = $variation['ID'];
				}
				$this->add_categories_to_variations($parent_variation_ids, $parent_to_variations,$porducts_under_custom_warning_level);
			}
		//wcosm_var_dump($porducts_under_custom_warning_level);	
		/* Format:
			array(2) {
				  ["ID"]=>
					string(3) "347"
					["post_title"]=>
					string(9) "Product 6"
					["post_type"]=>
					string(7) "product"
					["stock_value"]=>
					string(3) "yes"
					["stock_status"]=>
					string(7) "instock"
					["category"]=>
					string(25) "Category 3,Category 1.1.1"
					["product_type"]=>
					string(6) "simple"  //simple/varoable/variation
				  }
		*/
		//********** Default
		if($view_type == "all")
			$all_thresholds['default'] = 99999999999999999999;
		
		if($all_thresholds['default'] > 0)
		{
			
			
			//wcosm_var_dump($porducts_under_custom_warning_level);
			/* foreach($porducts_under_custom_warning_level as $item)
			{
				$ids_to_exclude[] = $item['ID'];
			} */
			/* if(isset($current_threshold['products']) && is_array($current_threshold['products']) && !empty($current_threshold['products']))
				foreach($current_threshold['products'] as $original_selected_prodcut_id)
					if(!in_array($original_selected_prodcut_id, $ids_to_exclude))
						$ids_to_exclude[] = $original_selected_prodcut_id; */
			/* wcosm_var_dump( empty($ids_to_exclude));	
			wcosm_var_dump( $ids_to_exclude); */	
			$ids_to_exclude = empty($ids_to_exclude) || $ids_to_exclude[0] == "" ? array(-1) : $ids_to_exclude;
			
			//if(!empty($ids_to_exclude))
			{
				$query = $shared_query." 
					  AND stock.meta_value <= {$all_thresholds['default']} 
					  AND products.ID NOT IN (".implode(",",$ids_to_exclude).") GROUP BY products.ID ".$max_items;
				
				$wpdb->query('SET group_concat_max_len=5000000'); 	
				$wpdb->query('SET SQL_BIG_SELECTS=1');
				$products_under_default_warning_level = $wpdb->get_results($query,ARRAY_A);
				$variations = $this->get_product_variations_by_warning_level($ids_to_exclude, null, $all_thresholds['default'], true);
				$parent_variation_ids = "";
				$parent_to_variations = array();
				foreach($variations as $variation)
				{
					$variation['product_type'] = "variation";
					$parent_variation_ids .= $parent_variation_ids != "" ? ",".$variation['post_parent'] : $variation['post_parent'];
					if(!isset($parent_to_variations[$variation['post_parent']]))
						$parent_to_variations[$variation['post_parent']] = array();
					$parent_to_variations[$variation['post_parent']][] = $variation['ID'];
					$products_under_default_warning_level[$variation['ID']] = $variation;
				}
				$this->add_categories_to_variations($parent_variation_ids, $parent_to_variations, $products_under_default_warning_level);
				foreach($products_under_default_warning_level as $item)
				{
					if($item['translation_original_id'] == null && !$wcosm_wpml_model->is_item_a_translation($item['ID'])) //WPML
						$porducts_under_custom_warning_level[$item['ID']] = $item;
				}
			}
		}	
		//wcosm_var_dump($products_under_default_warning_level);	
		$wcosm_wpml_model->switch_to_current_language();
		return $porducts_under_custom_warning_level;
	}
	
	public function is_product_under_custom_level($product_obj)
	{
		global $wcosm_wpml_model, $wcosm_option_model;
		$items = $product =  array();
		$all_thresholds = $wcosm_option_model->get_all_threshold_levels();
		$product['product_id'] = $product_obj->get_id();
		$product['variation_id'] = is_object($product) && $product->get_type() == 'variation' ? $product_obj->get_id() : 0;
		if($wcosm_wpml_model->is_active())
			{
				$product['product_id'] = $wcosm_wpml_model->get_original_id($product['product_id'], 'product', true);
				$product['variation_id'] = $product['variation_id'] != 0 ? $wcosm_wpml_model->get_original_id($product['variation_id'], 'product_variation', true) : 0;
			}
			$items[] = array('product_id' => $product['product_id'], 
							 'variation_id' =>  $product['variation_id'],
							 'product' => $product_obj );
		$result =  $this->filter_products_by_custom_stock_level($items, false);
		
		if(isset($result[$product['product_id']."-".$product['variation_id']]))
			return true;
		
		if($all_thresholds['default'] > 0 && $all_thresholds['default_email_notification_enabled'] == 'yes')
		{
			if($product_obj->managing_stock( ) && $product_obj->get_stock_quantity( ) < $all_thresholds['default'] && !isset($products_to_warn[$product['product_id']."-".$product['variation_id']]))
					return true;
		}
	}
	public function filter_products_by_custom_stock_level($products, $exclude_products_without_email_notification = true)
	{
		global $wcosm_option_model, $wpdb;
		$products_to_warn = array();
		$all_thresholds = $wcosm_option_model->get_all_threshold_levels();
		foreach($all_thresholds['thresholds'] as $current_threshold)
		{
			if($exclude_products_without_email_notification && $current_threshold['email_notification'] != 'yes' )
				continue;
			
			$additional_categories_ids = array();
			$current_threshold['products'] = !is_array($current_threshold['products']) ? array() : $current_threshold['products'];
			if(!empty($current_threshold['categories']))
			{
				if($current_threshold["categories_children"] == 'all_children')
				{
					foreach($current_threshold["categories"] as $current_category)
					{
						$args = array(
								'type'                     => 'post',
								'child_of'                 => $current_category,
								'parent'                   => '',
								'orderby'                  => 'name',
								'order'                    => 'ASC',
								'hide_empty'               => 1,
								'hierarchical'             => 1,
								'exclude'                  => '',
								'include'                  => '',
								'number'                   => '',
								'taxonomy'                 => 'product_cat',
								'pad_counts'               => false

							); 

							$categories = get_categories( $args );
							//wcosm_var_dump($categories);
							foreach($categories as $result)
								$additional_categories_ids[] = $result->term_id;
					}
					if(!empty($additional_categories_ids))
						$current_threshold["categories"] = array_merge($current_threshold["categories"],$additional_categories_ids);
				}
				$wpdb->query('SET group_concat_max_len=5000000'); 
				$wpdb->query('SET SQL_BIG_SELECTS=1');
				$get_prdocuts_from_categories_query = " SELECT DISTINCT GROUP_CONCAT(products.ID) as product_ids
															 FROM {$wpdb->posts} AS products 
															 INNER JOIN {$wpdb->term_relationships} AS term_rel ON term_rel.object_id = products.ID
															 INNER JOIN {$wpdb->term_taxonomy} AS term_tax ON term_tax.term_taxonomy_id = term_rel.term_taxonomy_id 
															 INNER JOIN {$wpdb->terms} AS terms ON terms.term_id = term_tax.term_id
															 WHERE  terms.term_id IN ('" . implode( "','", $current_threshold["categories"]). "')  
															 AND term_tax.taxonomy = 'product_cat' "; // GROUP BY terms.term_id;
								
				//recover product by category
				$additional_products_by_seleceted_categories = $wpdb->get_results($get_prdocuts_from_categories_query,ARRAY_A);
				$additional_products_by_seleceted_categories = isset($additional_products_by_seleceted_categories) ? array_shift($additional_products_by_seleceted_categories) : null;
				
				if(isset($additional_products_by_seleceted_categories) && isset($additional_products_by_seleceted_categories['product_ids']) )
				{
					$additional_products_by_seleceted_categories = strpos($additional_products_by_seleceted_categories['product_ids'],",") != false ? explode(",", $additional_products_by_seleceted_categories['product_ids']) : array($additional_products_by_seleceted_categories['product_ids']);
				}		
				if(isset($additional_products_by_seleceted_categories))
				 $current_threshold['products'] = array_merge($current_threshold['products'], $additional_products_by_seleceted_categories);
			}		
			
			foreach($products as $product)
			{
				$product['variation_id'] = !isset($product['variation_id']) ? 0 : $product['variation_id'];
				if($current_threshold["select_strategy"] == 'all' && (in_array($product['product_id'], $current_threshold['products']) || in_array($product['variation_id'], $current_threshold['products'])) ||
					   $current_threshold["select_strategy"] != 'all' && !in_array($product['product_id'], $current_threshold['products']) && ($product['variation_id'] != 0 || !in_array($product['variation_id'], $current_threshold['products'])) )
				{
					//In this way lower rules have higher priority
					if(isset($products_to_warn[$product['product_id']."-".$product['variation_id']]))
						unset($products_to_warn[$product['product_id']."-".$product['variation_id']]);
					
					if($product['product']->managing_stock( ) && $product['product']->get_stock_quantity( ) < $current_threshold['warning_level'])
					{
					   //Set email data
						$products_to_warn[$product['product_id']."-".$product['variation_id']] = array('product' => $product, 'rule' =>$current_threshold) ;//$product['variation_id'] != 0 ? $product['variation_id'] : $product['product_id'];
					}
				}
			}
		}
		
		return $products_to_warn;
	}
	public function check_and_send_warning_notification_for_low_stock_level($products, $order)
	{
		global $wcosm_option_model, $wpdb;
		//$product->get_stock_quantity( ); -> In case of variable products, if is stock managing is at parent level, will return parent stock level
		$all_thresholds = $wcosm_option_model->get_all_threshold_levels();
		$item_to_notify = array();
		$products_to_warn = $this->filter_products_by_custom_stock_level($products);
		
		//Default level
		if($all_thresholds['default'] > 0 && $all_thresholds['default_email_notification_enabled'] == 'yes')
		{
			foreach($products as $product)
				if($product['product']->managing_stock( ) && $product['product']->get_stock_quantity( ) < $all_thresholds['default'] && !isset($products_to_warn[$product['product_id']."-".$product['variation_id']]) /* !in_array($product['product_id'], $products_to_warn ) */)
				//Set email data
				$products_to_warn[$product['product_id']."-".$product['variation_id']] = array('product' => $product, 'rule' =>'none');//$product['variation_id'] != 0 ? $product['variation_id'] : $product['product_id'];
		}
		
		//Send email
		if(!empty($products_to_warn))
		{
			$email = new WCOSM_Email();
			$recipients = $all_thresholds['email_recipients'] ? $all_thresholds['email_recipients'] : get_bloginfo('admin_email');
			$email->trigger($recipients, $products_to_warn, $order);
		}
		
		return $products_to_warn;
	}
	public function get_complementry_ids($ids_to_exclude/* , $post_type = "product" */)
	{
		global $wpdb;
		$product_ids = array();
		foreach($ids_to_exclude as $product)
		{
			$product_ids[] = $product;
			/* if($product['variation_id'] != 0)
			{
				$product_ids[] = $product['variation_id'];
			}
			else
			{
				$product_ids[] = $product['product_id'];
			} */
		}
			
		$results = array();
		$wpdb->query('SET SQL_BIG_SELECTS=1');
		$query = "SELECT posts.ID, posts.post_type, posts.post_parent
				  FROM {$wpdb->posts} AS posts
				  WHERE 
				  (posts.post_type = 'product' OR posts.post_type = 'product_variation')
				  AND posts.ID NOT IN('".implode("','",$product_ids)."') ";
		$ids = $wpdb->get_results($query, ARRAY_A);
		if(isset($ids) && is_array($ids))
			foreach($ids as $id)
			{
				/* if($id['post_type'] == 'product')
					$results[] = array('product_id' => (int)$id['ID'], 'variation_id' => 0);
				else
					$results[] = array('product_id' => (int)$id['post_parent'], 'variation_id' => (int)$id['ID']); */
				$results[] = (int)$id['ID'];
			}
		return $results;
	}
	public function get_variation_complete_name($variation_id)
	{
		$error = false;
		$variation = null;
		try
		{
			$variation = new WC_Product_Variation($variation_id);
		}
		catch(Exception $e){return false; $error = true;}
		if($error) //no longer executed
    		try
    		{
    			$error = false;
    			$variation = new WC_Product($variation_id);
    			return $variation->get_title();
    		}catch(Exception $e){$error = true;}
		
		if($error)
			return "";
		
		$product_name = $variation->get_title()." - ";	
		if($product_name == " - ")
			return false;
		$attributes_counter = 0;
		
		foreach($variation->get_variation_attributes( ) as $attribute_name => $value)
		{
			
			if($attributes_counter > 0)
				$product_name .= ", ";
			$meta_key = urldecode( str_replace( 'attribute_', '', $attribute_name ) ); 
			
			$product_name .= " ".wc_attribute_label($meta_key).": ".$value;
			$attributes_counter++;
		}
		return $product_name;
	}
	public function add_categories_to_variations($parent_variation_ids, $parent_to_variations, &$product_array)
	{
		if($parent_variation_ids == "")
			return;
		global $wpdb;
		$wpdb->query('SET group_concat_max_len=5000000'); 
		$wpdb->query('SET SQL_BIG_SELECTS=1');
		$shared_query = "SELECT products.ID, products.post_title,  GROUP_CONCAT(term.name) AS category
					  FROM {$wpdb->posts} AS products 
					 
					  LEFT JOIN {$wpdb->term_relationships} AS category_relationship ON (products.ID = category_relationship.object_id) 
					  LEFT JOIN {$wpdb->term_taxonomy} AS taxonomy ON category_relationship.term_taxonomy_id = taxonomy.term_taxonomy_id AND taxonomy.taxonomy = 'product_cat' 
					  LEFT JOIN {$wpdb->terms} AS term ON (term.term_id = taxonomy.term_id) 
					
					  WHERE products.ID IN ({$parent_variation_ids}) 
					  GROUP BY products.ID ";
					  
		$result = $wpdb->get_results($shared_query, ARRAY_A);
		//wcosm_var_dump($result);
		if(isset($result))
			foreach($result as $item)
				foreach( $parent_to_variations[$item['ID']] as $variation_id)
					$product_array[$variation_id]['category'] = $item['category'];
			
	}
}
?>